/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef __GT_DMA_H__
#define __GT_DMA_H__

#include <linux/types.h>
#include "gt_plat_dev.h"
#include "gt_mm.h"
#include "hal_interface.h"

typedef void gt_dma_async_tx_desc;
typedef void gt_sg_table;
typedef void gt_sglist;
typedef void gt_dma_device;
typedef void gt_dma_chan;
typedef void gt_dma_pool;
typedef void gt_dma_slave_config;

typedef bool (*pfn_filter)(gt_dev* dev, int chan_id, void *chan_private, void *params);

extern unsigned long gt_dma_slave_config_size;

extern int gt_dma_cap_slave;
extern int gt_dma_cap_memcpy;
extern int gt_dma_cap_memset;
extern int gt_dma_cap_memset_sg;

extern int gt_dma_gddr2sys;
extern int gt_dma_sys2gddr;

enum gt_dma_data_direction {
	GT_DMA_BIDIRECTIONAL,
	GT_DMA_TO_DEVICE,
	GT_DMA_FROM_DEVICE,
	GT_DMA_NONE,
};

struct gt_dma_chan_filter_param
{
	void *params;
	bool (*pfn_filter)(gt_dev *dev, int chan_id, void *chan_private, void *params);
};

uint64_t gt_get_dma_mask(void *dev);

int gt_dma_set_mask_64(void *dev);

int gt_dma_set_mask_32(void *dev);

int gt_dma_get_ctl_flag(void);

gt_sg_table *gt_sg_table_alloc(void);

void gt_sg_table_free(gt_sg_table *sg);

uint64_t gt_sg_dma_address(gt_sglist *sg);

uint32_t gt_sg_dma_len(gt_sglist *sg);

int gt_dmaengine_slave_config(void *chan, bool mem_to_dev, uint64_t src_addr, uint64_t dst_addr);

void *gt_dma_map_page(gt_dev *dev, gt_page *pg, unsigned long pg_off,
						uint64_t align);

void gt_dma_unmap_page(gt_dev *dev, uint64_t addr, uint64_t align);

int gt_dma_mapping_error(gt_dev *dev, u64 dma_addr);

int gt_dma_map_sg(gt_dev *dev, gt_sg_table *sg_table);

void gt_dma_unmap_sg(gt_dev *dev, gt_sg_table *sg_table);

void gt_dma_sync_sg_for_device(gt_dev *dev, gt_sg_table *sg_table);

void gt_dma_sync_sg_for_cpu(gt_dev *dev, gt_sg_table *sg_table);

void gt_dma_sync_single_for_cpu(void *dev, dma_addr_t handle, uint64_t size);

void gt_dma_sync_single_for_device(void *dev, dma_addr_t handle, uint64_t size);

gt_dma_async_tx_desc *gt_dmaengine_prep_slave_sg(void *chan, gt_sg_table *sg_table,
		bool mem_to_dev, unsigned long flags);

gt_dma_async_tx_desc *gt_dmaengine_prep_memcpy(gt_dma_chan *chan, void *src, void *dst, int len);

gt_dma_async_tx_desc *gt_dmaengine_prep_memset(gt_dma_chan *chan, void *dest_addr, int value, uint64_t len);

gt_dma_async_tx_desc *gt_dmaengine_prep_memset_sg(gt_dma_chan *chan, gt_sg_table *sg_table, int value, int flags);

void gt_dmaengine_set_tx_desc_cb(gt_dma_async_tx_desc *desc, void (*cb)(void *), void *data);

int gt_dmaengine_submit(gt_dma_async_tx_desc *desc);

void gt_dma_async_issue_pending(void *chan);

int gt_dma_async_is_tx_complete(gt_dma_chan *chan, int cookie);

int gt_dmaengine_terminate_sync(void *chan);

gt_sglist *gt_sg_next(gt_sglist *sg);

gt_sglist *gt_sg_first(gt_sg_table *table);

unsigned int gt_sg_nents(gt_sg_table *table);

uint64_t gt_sg_phys(gt_sglist *sg);

void gt_cache_flush(void *dev,
		void *start,
		void *end,
		phys_addr_t phys_start,
		phys_addr_t phys_end);

void gt_cache_clean(void *dev,
		void *start,
		void *end,
		phys_addr_t phys_start,
		phys_addr_t phys_end);

void gt_cache_invalidate(void *dev,
		void *start,
		void *end,
		phys_addr_t phys_start,
		phys_addr_t phys_end);

#define gt_for_each_sg(sglist, sg, nr, __i) \
	for (__i = 0, sg = (sglist); __i < (nr); __i++, sg = gt_sg_next(sg))

gt_dma_chan *gt_request_hal_dma_chan(int dma_cap, void *param);

gt_dma_chan* gt_dma_request_channel(int dma_cap, pfn_filter filter, void *params);

void gt_dma_release_channel(gt_dma_chan *chan);

void gt_dmaengine_terminate_all(gt_dma_chan *chan);

const char *gt_dma_chan_name(gt_dma_chan *chan);

gt_dma_pool *gt_dmam_pool_create(const char *name, void *dev, uint64_t size,
									uint64_t align, uint64_t allocation);

uint64_t gt_dma_slave_config_get_dst(gt_dma_slave_config *cfg);

uint64_t gt_dma_slave_config_get_src(gt_dma_slave_config *cfg);

uint64_t GT_DMA_BIT_MASK(int n);

void *gt_alloc_dma_device(void *dev);

void gt_free_dma_device(void *dev, void *addr);

void *gt_alloc_dma_slave_config(void *dev);

void gt_free_dma_slave_config(void *dev, void *addr);

struct list_head* gt_get_dma_channels(void *dma_dev);

int gt_is_dma_mem2dev(int dir);

#define GT_DMA_MAP_SG_ATTRS(d, s, n, r) gt_dma_map_sg_attrs(d, s, n, r, 0)

#define GT_DMA_UNMAP_SG_ATTRS(d, s, n, r) gt_dma_unmap_sg_attrs(d, s, n, r, 0)

int gt_os_sg_alloc_table(gt_sg_table *table, unsigned int nents);

void gt_os_sg_free_table(gt_sg_table *table);

gt_sglist *gt_sgt_list_get(gt_sg_table *table);

unsigned int gt_sgt_nets_get(gt_sg_table *table);

void gt_sgt_nents_set(gt_sg_table *table, unsigned int nents);

unsigned int gt_sgt_orig_nents_get(gt_sg_table *table);

unsigned int gt_sg_nents_by_sgl(gt_sglist *sg);

gt_page *gt_sgl_get_page(gt_sglist *sg);

enum dma_data_direction gt_get_dma_direction(enum gt_dma_data_direction dir);

void gt_dma_sync_sg_for_cpu_with_dir(gt_dev *dev, gt_sglist *sgl, int nelems, enum gt_dma_data_direction dir);

void gt_dma_sync_sg_for_device_with_dir(gt_dev *dev, gt_sglist *sgl, int nelems, enum gt_dma_data_direction dir);

int gt_dma_map_sg_attrs(gt_dev *dev, gt_sglist *sg, int nents, enum gt_dma_data_direction dir, unsigned long attrs);

void gt_dma_unmap_sg_attrs(gt_dev *dev, gt_sglist *sg, int nents, enum gt_dma_data_direction dir, unsigned long attrs);

void gt_sgl_set_page(gt_sglist *sg, gt_page *page, unsigned int len, unsigned int offset);

void gt_sgl_set_dma_address(gt_sglist *sgl, u64 addr);

void gt_sgl_set_dma_len(gt_sglist *sgl, unsigned int len);

dma_addr_t gt_sgl_phys(gt_sglist *sg);

uint32_t gt_sgl_len(gt_sglist *sg);

uint32_t gt_sgl_offset(gt_sglist *sg);

void gt_map_set_vaddr(void* map, void* kptr);

void gt_map_current_lock(void);

void gt_map_current_unlock(void);

u64 gt_dma_get_mask(gt_dev *dev);

int gt_dma_set_mask(void *dev, uint64_t mask);

int gt_get_dma_chan_id(gt_dma_chan * chan);

bool gt_dmaengine_desc_test_reuse(gt_dma_async_tx_desc *tx);

int gt_dmaengine_desc_set_reuse(gt_dma_async_tx_desc *tx);

void gt_dmaengine_desc_clear_reuse(gt_dma_async_tx_desc *tx);
#endif
