/*
 * Copyright (c) Gengtu Technology Ltd. All Rights Reserved
 * Dual MIT/GPLv2
 *
 * The contents of this file are subject to the MIT license as set out below.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 ("GPL") in which case the provisions
 * of GPL are applicable instead of those above.
 *
 * If you wish to allow use of your version of this file only under the terms of
 * GPL, and not to allow others to use your version of this file under the terms
 * of the MIT license, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by GPL as set
 * out in the file called "GPL-COPYING" included in this distribution. If you do
 * not delete the provisions above, a recipient may use your version of this file
 * under the terms of either the MIT license or GPL.
 *
 * This License is also included in this distribution in the file called
 * "MIT-COPYING".
 *
 * EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
 * PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#ifndef __GT_FENCE_H__
#define __GT_FENCE_H__

#include "gt_lock.h"

typedef void gt_dma_fence;
typedef void gt_sync_file;
typedef void gt_dma_fence_array;

struct gt_kref;

struct gt_kref *gt_kref_alloc(void (*destroy)(void *data), void *data);

void gt_kref_free(struct gt_kref *kref);

int gt_kref_put(struct gt_kref *kref);

void gt_kref_get(struct gt_kref *kref);

void gt_dma_fence_set_error(gt_dma_fence *fence, int error);

int gt_dma_fence_get_error(gt_dma_fence *fence);

void gt_dma_fence_put(gt_dma_fence *fence);

gt_dma_fence *gt_dma_fence_get(gt_dma_fence *fence);

gt_dma_fence* gt_dma_fence_alloc(void);

void gt_dma_fence_init(gt_dma_fence *fence, const void *ops, gt_spinlock *lock, unsigned context, unsigned seqno);

int gt_dma_fence_signal(gt_dma_fence *fence);

bool gt_dma_fence_is_signaled(gt_dma_fence *fence);

void gt_dma_fence_free(gt_dma_fence* f);

void gt_os_free_dma_fence(gt_dma_fence *fence);

gt_dma_fence *gt_sync_file_get_fence(int fd);

uint64_t gt_dma_fence_context_alloc(unsigned num);

gt_dma_fence_array *gt_dma_fence_array_create(int num_fences, gt_dma_fence **fences, uint64_t context, unsigned seqno, bool signal_on_any);

gt_dma_fence *gt_dma_fence_array_get_base(gt_dma_fence_array *fence_array);

gt_sync_file *gt_sync_file_create(gt_dma_fence *fence);

void *gt_sync_file_get_file(gt_sync_file *sync_file);

#endif

