/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <linux/moduleparam.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <asm/div64.h>
#include <linux/bug.h>
#include <linux/err.h>
#include <linux/utsname.h>
#include <linux/kfifo.h>
#include <linux/module.h>
#include <linux/random.h>
#include <linux/bitops.h>
#include <linux/debugfs.h>
#include <linux/proc_fs.h>

#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0))
#include <linux/fs.h>
#endif

#include "gt_misc.h"
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0))
#include <linux/module_signature.h>
#else
#define PKEY_ID_PKCS7 2
#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0) */

const uint32_t gt_gfp_kernel = GFP_KERNEL;
GT_EXT_SYM(gt_gfp_kernel);

uint32_t gPVRDebugLevel = 0;
module_param(gPVRDebugLevel, uint, 0644);
MODULE_PARM_DESC(gPVRDebugLevel,
		"Sets the level of debug output (default 0x7)");
GT_EXT_SYM(gPVRDebugLevel);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 7, 0)) && defined(CONFIG_LOONGARCH)
#include <linux/kprobes.h>
typedef unsigned long (*kallsyms_lookup_name_t)(const char *name);
extern kallsyms_lookup_name_t gt_kallsyms_lookup_name_ptr;
static struct kprobe kp = {
	.symbol_name = "kallsyms_lookup_name"
};
#endif

void get_kallsyms_lookup_name_address(void)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 7, 0)) && defined(CONFIG_LOONGARCH)
	register_kprobe(&kp);
	gt_kallsyms_lookup_name_ptr = (kallsyms_lookup_name_t) kp.addr;
	unregister_kprobe(&kp);
#endif
}
GT_EXT_SYM(get_kallsyms_lookup_name_address);

int gt_memcmp(void *buf0, void *buf1, uint32_t size)
{
	return memcmp(buf0, buf1, size);
}
GT_EXT_SYM(gt_memcmp);

uint64_t gt_strlcat(char *dst, const char *src, uint64_t size)
{
	return strlcat(dst, src, size);
}
GT_EXT_SYM(gt_strlcat);

char *gt_strchr(const char *s, int c)
{
	return strchr(s, c);
}
GT_EXT_SYM(gt_strchr);

char *gt_strrchr(const char *s, int c)
{
	return strrchr(s, c);
}
GT_EXT_SYM(gt_strrchr);

char *gt_strstr(const char *s1, const char *s2)
{
	return strstr(s1, s2);
}
GT_EXT_SYM(gt_strstr);

int gt_sprintf(char *buf, size_t size, const char *fmt, ...)
{
	va_list arg_list;
	int count;

	va_start(arg_list, fmt);
	count = gt_vsnprintf(buf, size, fmt, arg_list);
	va_end(arg_list);

	return count;
}
GT_EXT_SYM(gt_sprintf);

int gt_vsnprintf(char *buf, uint64_t size, const char *fmt, gt_va_list args)
{
	return vsnprintf(buf, size, fmt, args);
}
GT_EXT_SYM(gt_vsnprintf);

char *gt_kasprintf(uint32_t gfp, const char *fmt, ...)
{
	va_list ap;
	char *p;

	va_start(ap, fmt);
	p = kvasprintf(gfp, fmt, ap);
	va_end(ap);

	return p;
}
GT_EXT_SYM(gt_kasprintf);

int gt_snprintf(char *buf, size_t size, const char *fmt, ...)
{
	va_list args;
	int i;

	va_start(args, fmt);
	i = vsnprintf(buf, size, fmt, args);
	va_end(args);

	return i;
}
GT_EXT_SYM(gt_snprintf);


uint64_t gt_strlen(const char *str)
{
	return strlen(str);
}
GT_EXT_SYM(gt_strlen);

uint64_t gt_strnlen(const char *str, uint64_t cnt)
{
	return strnlen(str, cnt);
}
GT_EXT_SYM(gt_strnlen);

int gt_strncmp(const char *str1, const char *str2, uint64_t size)
{
	return strncmp(str1, str2, size);
}
GT_EXT_SYM(gt_strncmp);

int gt_strcmp(const char *str1, const char *str2)
{
	return strcmp(str1, str2);
}
GT_EXT_SYM(gt_strcmp);

int gt_kstrtou32(const char *s, unsigned int base, uint32_t *res)
{
	return kstrtou32(s, base, res);
}
GT_EXT_SYM(gt_kstrtou32);

int gt_kstrtos32(const char *s, unsigned int base, int32_t *res)
{
	return kstrtos32(s, base, res);
}
GT_EXT_SYM(gt_kstrtos32);

int gt_kstrtou64(const char *s, unsigned int base, u64 *res)
{
	return kstrtou64(s, base, res);
}
GT_EXT_SYM(gt_kstrtou64);

int gt_kstrtos64(const char *s, unsigned int base, s64 *res)
{
	return kstrtos64(s, base, res);
}
GT_EXT_SYM(gt_kstrtos64);

char *gt_strsep(char **s, const char *ct)
{
	return strsep(s, ct);
}
GT_EXT_SYM(gt_strsep);

int gt_strcasecmp(const char *s1, const char *s2)
{
	return strcasecmp(s1, s2);
}
GT_EXT_SYM(gt_strcasecmp);

int gt_strlcpy(char *dest, const char *src, int size)
{
	return strlcpy(dest, src, size);
}
GT_EXT_SYM(gt_strlcpy);

char *gt_strncpy(char *dest, const char *src, int count)
{
	return strncpy(dest, src, count);
}
GT_EXT_SYM(gt_strncpy);

int gt_vsscanf(const char *buf, const char *fmt, gt_va_list args)
{
	return vsscanf(buf, fmt, args);
}
GT_EXT_SYM(gt_vsscanf);

int gt_sscanf(const char *buf, const char *fmt, ...)
{
	va_list args;
	int i;

	va_start(args, fmt);
	i = vsscanf(buf, fmt, args);
	va_end(args);

	return i;
}
GT_EXT_SYM(gt_sscanf);

bool gt_is_err(const void *ptr)
{
	return IS_ERR(ptr);
}
GT_EXT_SYM(gt_is_err);

bool gt_is_err_or_null(const void *ptr)
{
	return IS_ERR_OR_NULL(ptr);
}
GT_EXT_SYM(gt_is_err_or_null);

long gt_ptr_err(const void *ptr)
{
	return PTR_ERR(ptr);
}
GT_EXT_SYM(gt_ptr_err);

void gt_bug(void)
{
	BUG();
}
GT_EXT_SYM(gt_bug);

void gt_bug_on(int condition)
{
	BUG_ON(condition);
}
GT_EXT_SYM(gt_bug_on);

int gt_warn_on(int condition)
{
	return WARN_ON(condition);
}
GT_EXT_SYM(gt_warn_on);

int gt_warn_on_once(int condition)
{
	return WARN_ON_ONCE(condition);
}
GT_EXT_SYM(gt_warn_on_once);

const char *gt_utsname_sysname(void)
{
	return utsname()->sysname;
}
GT_EXT_SYM(gt_utsname_sysname);

const char *gt_utsname_release(void)
{
	return utsname()->release;
}
GT_EXT_SYM(gt_utsname_release);

const char *gt_utsname_version(void)
{
	return utsname()->version;
}
GT_EXT_SYM(gt_utsname_version);

const char *gt_utsname_machine(void)
{
	return utsname()->machine;
}
GT_EXT_SYM(gt_utsname_machine);

bool gt_kernel_version_ge(int x, int y, int z)
{
	return LINUX_VERSION_CODE >= KERNEL_VERSION(x, y, z);
}
GT_EXT_SYM(gt_kernel_version_ge);

bool gt_kernel_version_gt(int x, int y, int z)
{
	return LINUX_VERSION_CODE > KERNEL_VERSION(x, y, z);
}
GT_EXT_SYM(gt_kernel_version_gt);

bool gt_kernel_version_lt(int x, int y, int z)
{
	return LINUX_VERSION_CODE < KERNEL_VERSION(x, y, z);
}
GT_EXT_SYM(gt_kernel_version_lt);

void gt_atomic64_set(atomic64_t *v, long long i)
{
	atomic64_set(v, i);
}
GT_EXT_SYM(gt_atomic64_set);

void gt_atomic64_add(long long i, atomic64_t *v)
{
	atomic64_add(i, v);
}
GT_EXT_SYM(gt_atomic64_add);

void gt_atomic64_sub(long long i, atomic64_t *v)
{
	atomic64_sub(i, v);
}
GT_EXT_SYM(gt_atomic64_sub);

long long gt_atomic64_read(const atomic64_t *v)
{
	return atomic64_read(v);
}
GT_EXT_SYM(gt_atomic64_read);

void gt_kill_fasync(void **fp)
{
	kill_fasync((struct fasync_struct **)fp, SIGIO, POLL_IN);
}
GT_EXT_SYM(gt_kill_fasync);

unsigned int gt_kfifo_len(void *fifo)
{
	return kfifo_len((struct kfifo *)fifo);
}
GT_EXT_SYM(gt_kfifo_len);

int gt_kfifo_is_full(void *fifo)
{
	return kfifo_is_full((struct kfifo *)fifo);
}
GT_EXT_SYM(gt_kfifo_is_full);

void gt_kfifo_reset(void *fifo)
{
	kfifo_reset((struct kfifo *)fifo);
}
GT_EXT_SYM(gt_kfifo_reset);

unsigned int gt_kfifo_in_spinlocked(void *kfifo, void *buf, unsigned long n, void *lock)
{
	return kfifo_in_spinlocked((struct kfifo *)kfifo, buf, n, (spinlock_t *)lock);
}
GT_EXT_SYM(gt_kfifo_in_spinlocked);

unsigned int gt_kfifo_out_spinlocked(void *kfifo, void *buf, unsigned long n, void *lock)
{
	return kfifo_out_spinlocked((struct kfifo *)kfifo, buf, n, (spinlock_t *)lock);
}
GT_EXT_SYM(gt_kfifo_out_spinlocked);

unsigned long gt_simple_strtoul(const char *cp, char **endp, unsigned int base)
{
	return simple_strtoull(cp, endp, base);
}
GT_EXT_SYM(gt_simple_strtoul);

void gt_this_module_put(void)
{
	module_put(THIS_MODULE);
}
GT_EXT_SYM(gt_this_module_put);

void gt_mb(void)
{
	mb();
}
GT_EXT_SYM(gt_mb);

void gt_smp_mb(void)
{
	smp_mb();
}
GT_EXT_SYM(gt_smp_mb);

void gt_smp_wmb(void)
{
	smp_wmb();
}
GT_EXT_SYM(gt_smp_wmb);

void gt_smp_rmb(void)
{
	smp_rmb();
}
GT_EXT_SYM(gt_smp_rmb);

void gt_cpu_relax(void)
{
	cpu_relax();
}
GT_EXT_SYM(gt_cpu_relax);

bool gt_is_aligned(uint64_t val, int aligned)
{
	return IS_ALIGNED((unsigned long)val, aligned);
}
GT_EXT_SYM(gt_is_aligned);

uint32_t gt_align(uint64_t val, uint64_t aligned)
{
	return ALIGN(val, aligned);
}
GT_EXT_SYM(gt_align);

int gt_num_possible_cpus(void)
{
	return num_possible_cpus();
}
GT_EXT_SYM(gt_num_possible_cpus);

void gt_get_random_bytes(void *buf, int nbytes)
{
	get_random_bytes(buf, nbytes);
}
GT_EXT_SYM(gt_get_random_bytes);

#if defined (__i386__) || defined(__x86_64__)
extern unsigned int tsc_khz;
unsigned int gt_tsc_khz(void)
{
	return tsc_khz;
}
GT_EXT_SYM(gt_tsc_khz);
#endif

uint64_t gt_div_round_up(uint64_t val, uint64_t div)
{
	return DIV_ROUND_UP(val, div);
}
GT_EXT_SYM(gt_div_round_up);

bool gt_test_and_set_bit(unsigned long nr, volatile unsigned long *addr)
{
	return test_and_set_bit(nr, addr);
}
GT_EXT_SYM(gt_test_and_set_bit);

bool gt_test_and_clear_bit(unsigned long nr, volatile unsigned long *addr)
{
	return test_and_clear_bit(nr, addr);
}
GT_EXT_SYM(gt_test_and_clear_bit);

void gt_set_bit(unsigned long nr, volatile unsigned long *addr)
{
	set_bit(nr, addr);
}
GT_EXT_SYM(gt_set_bit);

void gt_clear_bit(unsigned long nr, volatile unsigned long *addr)
{
	clear_bit(nr, addr);
}
GT_EXT_SYM(gt_clear_bit);

int gt_ffz(unsigned long word)
{
	return ffz(word);
}
GT_EXT_SYM(gt_ffz);

uint32_t gt_cpu_to_le32(uint32_t val)
{
	return cpu_to_le32(val);
}
GT_EXT_SYM(gt_cpu_to_le32);

uint32_t gt_le32_to_cpu(uint32_t val)
{
	return le32_to_cpu(val);
}
GT_EXT_SYM(gt_le32_to_cpu);

uint64_t gt_cpu_to_le64(uint64_t val)
{
	return cpu_to_le64(val);
}
GT_EXT_SYM(gt_cpu_to_le64);

uint32_t gt_le64_to_cpu(uint64_t val)
{
	return le64_to_cpu(val);
}
GT_EXT_SYM(gt_le64_to_cpu);

uint32_t gt_upper_32_bits(uint64_t val)
{
	return upper_32_bits(val);
}
GT_EXT_SYM(gt_upper_32_bits);

unsigned long gt_ffs(unsigned long val)
{
	return __ffs(val);
}
GT_EXT_SYM(gt_ffs);

int gt_kernel_config_numa(void)
{
#if defined(CONFIG_NUMA)
	return 1;
#else
	return 0;
#endif
}
GT_EXT_SYM(gt_kernel_config_numa);

unsigned int gt_get_pkey_id_pkcs7(void)
{
	return PKEY_ID_PKCS7;
}
GT_EXT_SYM(gt_get_pkey_id_pkcs7);

unsigned int gt_hweight8(unsigned int w)
{
	return hweight8(w);
}
GT_EXT_SYM(gt_hweight8);

gt_dentry* gt_debugfs_or_procfs_create_dir(char *pszName, gt_dentry* parent)
{
#if defined(CONFIG_DEBUG_FS)
	return debugfs_create_dir(pszName, parent);
#elif defined(CONFIG_PROC_FS)
	return proc_mkdir(pszName, parent);;
#endif /* defined(CONFIG_DEBUG_FS) || defined(CONFIG_PROC_FS) */
}
GT_EXT_SYM(gt_debugfs_or_procfs_create_dir);

void gt_debugfs_or_procfs_remove_dir(gt_dentry* psEntry)
{
#if defined(CONFIG_DEBUG_FS)
	debugfs_remove(psEntry);
#elif defined(CONFIG_PROC_FS)
	proc_remove(psEntry);
#endif /* defined(CONFIG_DEBUG_FS) || defined(CONFIG_PROC_FS) */
}
GT_EXT_SYM(gt_debugfs_or_procfs_remove_dir);

gt_dentry* gt_debugfs_or_procfs_create_file(gt_dev *dev, char *pszName, int uimode, gt_dentry* parent, const gt_file_operations *file_ops)
{
#if defined(CONFIG_DEBUG_FS)
	return debugfs_create_file(pszName, uimode, parent, dev, file_ops);
#elif defined(CONFIG_PROC_FS)
	return proc_create_data(pszName, uimode, parent, file_ops, dev);
#endif /* defined(CONFIG_DEBUG_FS) || defined(CONFIG_PROC_FS) */
}
GT_EXT_SYM(gt_debugfs_or_procfs_create_file);

void gt_debugfs_or_procfs_remove_file(gt_dentry* psEntry, char *pszName, gt_dentry* parent)
{
#if defined(CONFIG_DEBUG_FS)
	debugfs_remove(psEntry);
#elif defined(CONFIG_PROC_FS)
	remove_proc_entry(pszName, parent);
#endif /* defined(CONFIG_DEBUG_FS) || defined(CONFIG_PROC_FS) */
}
GT_EXT_SYM(gt_debugfs_or_procfs_remove_file);

void *gt_get_dfs_file(gt_inode *inode)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5,17,0))
	return PDE_DATA((struct inode *)inode);
#else
	return pde_data((struct inode *)inode);
#endif
}
GT_EXT_SYM(gt_get_dfs_file);
