/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef __GT_MISC_H__
#define __GT_MISC_H__

#include <linux/types.h>
#include <linux/export.h>
#include "gt_fs.h"
#include "gt_plat_dev.h"

/* for compatibility: move few math functions from gt_misc.c to gt_math.c */
#include "gt_math.h"

#if defined(SEPARATE_BUILD) || defined(CONFIG_DRM_GT_APU) || defined(CONFIG_GTEST_GT_DMA)
 #define GT_EXT_SYM(f) EXPORT_SYMBOL(f)
 #define GT_EXT_TRACEPOINT_SYM(f) EXPORT_TRACEPOINT_SYMBOL(f)
#else
 #define GT_EXT_SYM(f)
 #define GT_EXT_TRACEPOINT_SYM(f)
#endif

typedef __builtin_va_list gt_va_list;
typedef void gt_file_operations;

#define STRINGIFY_1(x) #x
#define STRINGIFY(x) STRINGIFY_1(x)

extern uint32_t gPVRDebugLevel;

extern const uint32_t gt_gfp_kernel;

int gt_memcmp(void *buf0, void *buf1, uint32_t size);

uint64_t gt_strlcat(char *dst, const char *src, uint64_t size);

char *gt_strchr(const char *s, int c);

char *gt_strrchr(const char *s, int c);

char *gt_strstr(const char *s1, const char *s2);

int gt_sprintf(char *buf, size_t size, const char *fmt, ...);

int gt_vsnprintf(char *buf, uint64_t size, const char *fmt, gt_va_list args);

uint64_t gt_strlen(const char *str);

uint64_t gt_strnlen(const char *str, uint64_t cnt);

int gt_strcmp(const char *str1, const char *str2);

int gt_strncmp(const char *str1, const char *str2, uint64_t size);

int gt_kstrtou32(const char *s, unsigned int base, uint32_t *res);

int gt_kstrtos32(const char *s, unsigned int base, int32_t *res);

int gt_kstrtou64(const char *s, unsigned int base, u64 *res);

int gt_kstrtos64(const char *s, unsigned int base, s64 *res);

char *gt_strsep(char **s, const char *ct);

int gt_strcasecmp(const char *s1, const char *s2);

int gt_strlcpy(char *dest, const char *src, int size);

char *gt_strncpy(char *dest, const char *src, int count);

int gt_vsscanf(const char *buf, const char *fmt, gt_va_list args);

int gt_sscanf(const char *buf, const char *fmt, ...);

bool gt_is_err(const void *ptr);

bool gt_is_err_or_null(const void *ptr);

long gt_ptr_err(const void *ptr);

void gt_bug(void);

void gt_bug_on(int condition);

int gt_warn_on(int condition);

int gt_warn_on_once(int condition);

void gt_dump_stack(void);

const char *gt_utsname_sysname(void);

const char *gt_utsname_release(void);

const char *gt_utsname_version(void);

const char *gt_utsname_machine(void);

bool gt_kernel_version_ge(int x, int y, int z);

bool gt_kernel_version_gt(int x, int y, int z);

bool gt_kernel_version_lt(int x, int y, int z);

void gt_atomic64_set(atomic64_t *v, long long i);

void gt_atomic64_add(long long i, atomic64_t *v);

void gt_atomic64_sub(long long i, atomic64_t *v);

long long gt_atomic64_read(const atomic64_t *v);

void gt_kill_fasync(void **fp);

unsigned int gt_kfifo_len(void *fifo);

int gt_kfifo_is_full(void *fifo);

void gt_kfifo_reset(void *fifo);

unsigned int gt_kfifo_in_spinlocked(void *kfifo, void *buf, unsigned long n, void *lock);

unsigned int gt_kfifo_out_spinlocked(void *kfifo, void *buf, unsigned long n, void *lock);

char *gt_kasprintf(uint32_t gfp, const char *fmt, ...);

int gt_snprintf(char *buf, size_t size, const char *fmt, ...);

unsigned long gt_simple_strtoul(const char *cp, char **endp, unsigned int base);

void gt_this_module_put(void);

void gt_mb(void);

void gt_smp_mb(void);

void gt_smp_wmb(void);

void gt_smp_rmb(void);

void gt_cpu_relax(void);

bool gt_is_aligned(uint64_t val, int aligned);

uint32_t gt_align(uint64_t val, uint64_t aligned);

int gt_num_possible_cpus(void);

int gt_get_unused_fd_flags_o_cloexec(void);

#if defined (__i386__) || defined(__x86_64__)
unsigned int gt_tsc_khz(void);
#endif

uint64_t gt_div_round_up(uint64_t val, uint64_t div);

bool gt_test_and_set_bit(unsigned long nr, volatile unsigned long *addr);

bool gt_test_and_clear_bit(unsigned long nr, volatile unsigned long *addr);

void gt_set_bit(unsigned long nr, volatile unsigned long *addr);

void gt_clear_bit(unsigned long nr, volatile unsigned long *addr);

int gt_ffz(unsigned long word);

uint32_t gt_cpu_to_le32(uint32_t val);

uint32_t gt_le32_to_cpu(uint32_t val);

uint64_t gt_cpu_to_le64(uint64_t val);

uint32_t gt_le64_to_cpu(uint64_t val);

uint32_t gt_upper_32_bits(uint64_t val);

unsigned long gt_ffs(unsigned long val);

int gt_kernel_config_numa(void);

unsigned int gt_get_pkey_id_pkcs7(void);

unsigned int gt_hweight8(unsigned int w);

void get_kallsyms_lookup_name_address(void);

gt_dentry* gt_debugfs_or_procfs_create_dir(char *pszName, gt_dentry* parent);

void gt_debugfs_or_procfs_remove_dir(gt_dentry* psEntry);

gt_dentry* gt_debugfs_or_procfs_create_file(gt_dev *dev, char *pszName, int uimode, gt_dentry* parent, const gt_file_operations *file_ops);

void gt_debugfs_or_procfs_remove_file(gt_dentry* psEntry, char *pszName, gt_dentry* parent);

void *gt_get_dfs_file(gt_inode *inode);
#endif
