/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef __GT_MM_H__
#define __GT_MM_H__

#include <linux/types.h>

extern unsigned long gt_page_size;
extern uint8_t  gt_page_shift;
extern int gt_page_struct_size;

#define GT_PAGE_IDX_SIZE(idx) (gt_page_struct_size * (idx))

typedef void gt_page;
typedef void gt_pgprot;
typedef void gt_vm_area;
typedef void gt_kmem_cache;
typedef void gt_mm_struct;
struct gt_shrinker;
typedef void gt_shrink_control;
typedef void gt_gen_pool;
typedef void gt_gen_pool_chunk;
typedef void gt_memblock;
typedef struct
{
#if defined(UNDER_WDDM) || defined(WINDOWS_WDF)
	uintptr_t ui_addr;
#elif defined(__linux__) && defined(__KERNEL__)
	phys_addr_t ui_addr;
#else
	uint64_t ui_addr;
#endif
} GT_CPU_PHYADDR;

void gt_pgprot_init(void);

unsigned long gt_pgprot_val(gt_pgprot *pgprot);

void *gt_kmalloc_kernel(uint64_t size);

void *gt_krealloc_kernel(const void *p, size_t new_size);

void *gt_kmalloc_atomic(uint64_t size);

void *gt_kzalloc_kernel(uint64_t size);

void *gt_kmalloc_array(size_t n, size_t size);

void *gt_kzalloc_array(size_t n, size_t size);

void *gt_kmalloc_page_array(size_t n);

void *gt_kzalloc_page_array(size_t n);

void gt_kfree(const void *);

uint64_t gt_ksize(const void *addr);

void *gt_vmalloc(uint64_t size);

void *gt_vzalloc(uint64_t size);

void gt_vfree(const void *);

bool gt_is_vmalloc_addr(const void *addr);

bool gt_is_kmalloc_addr(const void *addr);

void *gt_kvmalloc_kernel(uint64_t size);

void gt_kvfree(const void *addr);

gt_page* gt_vmalloc_to_page(const void *addr);

void gt_set_page_reserved(gt_page *page);

void gt_clear_page_reserved(gt_page *page);

void gt_set_vmalloc_pages_reserved(void *addr, uint64_t size);

void gt_clear_vmalloc_pages_reserved(void *addr, uint64_t size);

void *gt_devm_kmalloc_kernel(void *dev, uint64_t size);

void *gt_devm_kzalloc_kernel(void *dev, uint64_t size);

void *gt_devm_kcalloc_kernel(void *dev, uint64_t n, uint64_t size);

void gt_devm_kfree(void *dev, void *addr);

void *gt_kmemdup_kernel(const void *src, uint64_t len);

uint64_t gt_page_align(uint64_t addr);

struct list_head* gt_page_lru(gt_page *page);

uint32_t gt_get_order(uint64_t size);

uint32_t gt_compound_order(gt_page *page);

gt_page *gt_alloc_pages(uint32_t flags, uint32_t order);

void gt_page_set_mapping(gt_page *page, void* ptr);

gt_page *gt_page_inc(gt_page *page, uint64_t order);

gfp_t gt_page_flag_comp(void);

gfp_t gt_page_flag_nowarn(void);

gfp_t gt_page_flag_kernel(void);

gt_page *gt_alloc_gtt_highmem_pages(uint32_t order);

gt_page *gt_virt_to_page(void *addr);

unsigned long gt_offset_in_page(void *addr);

uint64_t gt_virt_to_phys(void *addr);

void gt_split_page(gt_page *page, unsigned int order);

uint32_t gt_alloc_uma_flags(void *dev);

uint32_t gt_alloc_osmem_flags(bool zero, void *dev);

uint32_t gt_alloc_osgtt_flags(bool zero, void *dev);

uint32_t gt_gfp_reclaim(void);

uint32_t gt_gfp_noretry(void);

void gt_free_pages(gt_page *pages, uint32_t order);

uint64_t gt_page_to_phys(gt_page *page);

gt_page *gt_phys_to_page(uint64_t addr);

void *gt_page_address(const gt_page *page);

int gt_set_memory_wb(unsigned long addr, int numpages);

int gt_set_memory_wc(unsigned long addr, int numpages);

int gt_set_memory_uc(unsigned long addr, int numpages);

int gt_set_pages_array_uc(gt_page **pages, int nums);

int gt_set_pages_array_wc(gt_page **pages, int nums);

int gt_set_pages_array_wb(gt_page **pages, int nums);

void gt_pgprot_noncached(gt_pgprot *old_prot, gt_pgprot *prot_res);

void gt_pgprot_cached(gt_pgprot *old_prot, gt_pgprot *prot_res);

void gt_pgprot_writecombine(gt_pgprot *old_prot, gt_pgprot *prot_res);

gt_pgprot *gt_get_gt_page_kernel(void);

unsigned long gt_get_vm_map_flag(void);

void *gt_vmap(gt_page **pages, unsigned int cnt, unsigned long flags, gt_pgprot *prot);

void gt_vunmap(void *addr, unsigned int count, gt_pgprot *prot);

gt_page *gt_page_idx(gt_page* page, uint32_t index);

void *gt_kmap(gt_page *page);

void gt_kunmap(gt_page *page);

void *gt_kmap_atomic(gt_page *page);

void gt_kunmap_atomic(void *addr);

void gt_current_mmap_write_lock(void);

void gt_current_mmap_write_unlock(void);

void gt_current_mmap_read_lock(void);

void gt_current_mmap_read_unlock(void);

gt_vm_area *gt_find_vma(uint64_t address);

void gt_unmap_mapping_range(gt_vm_area *vma, uint64_t addr, uint32_t size, int even_cows);

uint64_t gt_phys_to_pfn(uint64_t addr, uint64_t flags);

uint64_t gt_pfn_down(uint64_t addr);

uint64_t gt_page_to_pfn(gt_page *page);

gt_page *gt_pfn_to_page(uint64_t pfn);

int gt_pfn_valid(uint64_t pfn);

int gt_page_count(gt_page *page);

gt_mm_struct *gt_get_current_mm_struct(void);

void gt_vm_area_pgoff_reset(gt_vm_area *vma);

unsigned long gt_vm_area_start(gt_vm_area *vma);

unsigned long gt_vm_area_end(gt_vm_area *vma);

void gt_vm_area_set_mixed_flag(gt_vm_area *vma);

int gt_vm_area_is_mixed_flag(gt_vm_area *vma);

int gt_vm_area_has_write_flag(gt_vm_area *vma);

int gt_vm_area_has_share_flag(gt_vm_area *vma);

int gt_vm_area_has_read_flag(gt_vm_area *vma);

void gt_vm_get_page_prot(gt_vm_area *vma, gt_pgprot *prot);

void gt_vm_set_page_prot(gt_vm_area *vma, gt_pgprot *prot);

void gt_vm_area_set_io_flag(gt_vm_area *vma);

void gt_vm_area_set_dontdump_flag(gt_vm_area *vma);

void gt_vm_area_set_dontexpand_flag(gt_vm_area *vma);

void gt_vm_area_set_dontcopy_flag(gt_vm_area *vma);

void gt_vm_area_set_pfnmap_flag(gt_vm_area *vma);

void gt_vm_area_unset_pfnmap_flag(gt_vm_area *vma);

void gt_vm_area_set_private_data(gt_vm_area *vma, void* data);

void* gt_vm_area_get_private_data(gt_vm_area *vma);

void gt_vm_area_set_pgoff(gt_vm_area *vma, unsigned long pgoff);

unsigned long gt_vm_area_len(gt_vm_area *vma);

int gt_vm_insert_mixed(gt_vm_area *vma, uint64_t addr, uint64_t pfn);

int gt_vm_insert_page(gt_vm_area *vma, uint64_t addr, gt_page *page);

int gt_vm_insert_pfn(gt_vm_area *vma, unsigned long addr, unsigned long pfn);

int gt_set_page_dirty_lock(gt_page *page);

int gt_set_page_dirty(gt_page *page);

void gt_put_page(gt_page *page);

void gt_get_page(gt_page *page);

int gt_get_user_pages_fast(uint64_t addr, int nr_pages, int write, gt_page **pages);

long gt_get_user_pages(uint64_t addr, int nr_pages, unsigned int flag, gt_page **pages, void* vmas);

int gt_cache_line_size(void);

void *gt_ioremap(uint64_t addr, uint64_t size);

void *gt_ioremap_nocache(uint64_t addr, uint64_t size);

void *gt_ioremap_cache(uint64_t addr, uint64_t size);

void *gt_ioremap_wc(uint64_t addr, uint64_t size);

void *gt_ioremap_wc_portable(uint64_t addr, uint64_t size);

void gt_iounmap(void *ptr);

int gt_access_ok(const void __user *addr, unsigned long size);

unsigned long gt_copy_from_user(void *to, const void __user *from, unsigned long bytes);

unsigned long gt_copy_to_user(void __user *to, const void *from, unsigned long bytes);

gt_kmem_cache *gt_kmem_cache_create(const char *name, unsigned int size, unsigned int align, unsigned int flags, void (*ctor)(void *));

void gt_kmem_cache_destroy(gt_kmem_cache *s);

void *gt_kmem_cache_alloc_kernel(gt_kmem_cache *s);
void gt_kmem_cache_free(gt_kmem_cache *s, void *objp);

void *gt_dma_alloc_coherent(void *dev, uint64_t size, dma_addr_t *dma_handle, uint32_t gfp);

void *gt_dma_alloc_coherent_kernel(void *dev, uint64_t size, dma_addr_t *dma_handle);

void gt_dma_free_coherent(void *dev, uint64_t size, void *cpu_addr, dma_addr_t dma_handle);

void gt_get_os_ram_stats(uint64_t *total_size, uint64_t *free_size);

struct gt_shrinker *gt_register_shrinker(
		unsigned long (*count_objects)(struct gt_shrinker *, gt_shrink_control *sc),
		unsigned long (*scan_objects)(struct gt_shrinker *, gt_shrink_control *sc));

void gt_unregister_shrinker(struct gt_shrinker *s);

unsigned long gt_get_shrinker_obj_nr_to_scan(gt_shrink_control *sc);

void gt_set_vm_area_priv(gt_vm_area *vma, void *priv);

bool gt_is_valid_pmr_vaddr(gt_vm_area *vma, unsigned long vaddr,
		unsigned long size, void *pmr);

int gt_remap_pfn_range(gt_vm_area *vma, unsigned long addr,
	unsigned long pfn, unsigned long size);

void *gt_memset(void *s, int c, size_t count);

void *gt_memcpy(void *dest, const void *src, size_t count);

void gt_memcpy_fromio(void *to, const volatile void __iomem *from, size_t count);

void gt_memcpy_toio(volatile void __iomem *to, const void *from, size_t count);

void gt_memset_io(volatile void __iomem *a, int b, size_t c);

void gt_memset_io_portable(void *s, int c, size_t count);

unsigned long gt_get_dma32_zone_pages(void);

unsigned long gt_get_normal_high_zone_total_pages(void);

uint64_t gt_get_zonedma32_totalsize(void);

uint64_t gt_get_zonedma32_freesize(void);

bool gt_mem_init_on_alloc(void);

bool gt_os_phys_non_contig_supported(void);

void gt_os_unmap_phys_array_to_lin(void *addr, void *priv_data);

void* gt_os_map_phys_array_to_lin(GT_CPU_PHYADDR* page_phy_addr, uint32_t pages_count, void ** pp_vlin_addr, void **pp_vpriv_data);

void gt_set_prot_default_page_kernel(gt_pgprot *prot);

void gt_flush_cache_range(gt_vm_area* vma, unsigned long addr_start, unsigned long addr_end);

unsigned long gt_gen_pool_alloc(gt_gen_pool * pool, size_t size);

int gt_gen_pool_add_virt(gt_gen_pool * pool, unsigned long virt_start, phys_addr_t phys, size_t size, int nid);

void gt_gen_pool_free(gt_gen_pool * pool, unsigned long start, size_t size);

gt_gen_pool *gt_gen_pool_create(int order, int nid);

phys_addr_t gt_gen_pool_virt_to_phys(gt_gen_pool * pool, unsigned long start);

int gt_gen_pool_add(gt_gen_pool * pool, unsigned long addr, size_t size, int nid);

void gt_gen_pool_destroy(gt_gen_pool * pool);

unsigned long gt_gen_pool_chunk_start_addr(gt_gen_pool_chunk * chunk);

unsigned long gt_gen_pool_chunk_end_addr(gt_gen_pool_chunk * chunk);

unsigned long gt_gen_pool_avail_size(gt_gen_pool * pool);

gt_page *gt_alloc_pages_by_node(uint32_t node_id, uint32_t flags, uint32_t order);
uint64_t gt_get_zone_normal_totalsize_by_node(uint32_t node_id);
uint64_t gt_get_zone_normal_freesize_by_node(uint32_t node_id);

gt_page* gt_get_first_page(struct list_head* head);
void gt_pages_insert(gt_page* page, struct list_head* head);

void gt_get_next_mem_pfn_range(int *idx, int nid,
				unsigned long *out_start_pfn,
				unsigned long *out_end_pfn, int *out_nid, gt_memblock* memblock);
unsigned long gt_kallsyms_lookup_name(const char *name);
void gt_put_user_pages(gt_page **pages, unsigned long npages);

int gt_get_user(void *dest, const void __user *src, bool is_u32);

int gt_put_user(const void *src, void __user *dest);
#endif
