/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef __GT_TASK_H__
#define __GT_TASK_H__

#include <linux/types.h>
#include "gt_misc.h"

typedef void gt_task;
typedef void gt_workqueue;
typedef void gt_work;
typedef void gt_dwork;
typedef void gt_pid;

#define GT_WQ_UNBOUND (1 << 1)
#define GT_WQ_FREEZABLE (1 << 2)
#define GT_WQ_HIGHPRI             (1 << 4)
#define GT_WQ_MAX_ACTIVE          512
#define GT_WQ_MAX_UNBOUND_PER_CPU 4
/* unbound wq's aren't per-cpu, scale max_active according to #cpus */
#define GT_WQ_UNBOUND_MAX_ACTIVE	\
	gt_max_type_int(GT_WQ_MAX_ACTIVE, gt_num_possible_cpus() * GT_WQ_MAX_UNBOUND_PER_CPU)

#define gt_might_sleep_if(cond) do { if (cond) gt_might_sleep(); } while (0)

gt_task *gt_get_current_task(void);

void gt_enable_oom_killer(void);

void gt_disable_oom_killer(void);

void gt_task_set_current_state(long state);

void gt_task_schedule(void);

void gt_task_set_user_nice(gt_task *task, long nice);

int gt_task_get_current_nice(void);

int gt_task_stop(gt_task *task);

void gt_tasklet_kill(gt_task *task);

void gt_do_exit(long code);

uint64_t gt_task_size(void);

uint32_t gt_task_id(void);

uint32_t gt_task_pid(void);

uint32_t gt_task_pid_vnr(void);

uint32_t gt_task_virtual_id(void);

gt_pid *gt_find_vpid(uint32_t nr);

char *gt_get_task_name_by_pid(uint32_t nr);

int gt_kill_pid(gt_pid *pid, int sig, int priv);

char *gt_task_name(void);

bool gt_task_is_in_kernel(void);

#ifdef CONFIG_FAULT_INJECTION
void gt_task_set_current_make_fail(bool val);
#endif

bool gt_set_freezable(void);

bool gt_kthread_freezable_should_stop(void);

void gt_get_task_struct(gt_task *task);

void gt_put_task_struct(gt_task *task);

gt_task *gt_task_create(int (*func)(void *data), void *data, const char name[]);

gt_workqueue *gt_create_freezable_wq(const char *name);

gt_workqueue *gt_create_workqueue(const char *name);

gt_workqueue *gt_create_singlethread_wq(const char *name);

gt_workqueue *gt_alloc_workqueue(const char *name, unsigned int flags, int max_active);

void gt_destroy_workqueue(gt_workqueue *wq);

void *gt_work_alloc(void (*func)(void *data), void *data);

void gt_work_destroy(gt_work *work);

void *gt_dwork_alloc(void (*func)(void *data), void *data);

int gt_queue_dwork(gt_workqueue *wq, gt_dwork *dwk, unsigned int msec);

int gt_schedule_dwork(gt_dwork *dwk, unsigned int msec);

void gt_flush_scheduled_work(void);

int gt_mod_dwork(gt_dwork *dwk, unsigned int msec);

int gt_cancel_dwork(gt_dwork *dwk);

int gt_cancel_dwork_sync(gt_dwork *dwk);

void gt_dwork_destroy(void *dwork);

int gt_queue_work(gt_workqueue *wq, gt_work *wk);

int gt_queue_work_on(int cpu, gt_workqueue *wq, void *wk);

bool gt_schedule_work(void *wk);

bool gt_cancel_work_sync(void *wk);

void gt_flush_workqueue(gt_workqueue *wq);

bool gt_flush_work(gt_work *work);

signed long  gt_schedule_timeout(signed long timeout);

void gt_might_sleep(void);
unsigned int gt_work_busy(gt_work *work);
#endif

