/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#include <linux/version.h>
#include <linux/dma-buf.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0))
#include <linux/iosys-map.h>
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 11, 0))
#include <linux/dma-buf-map.h>
#endif
#include "gt_misc.h"
#include "gt_dma_buf.h"
#include "gt_srvkm.h"

static int gt_pmr_dma_buf_ops_attach(struct dma_buf *buf,
#if ((LINUX_VERSION_CODE < KERNEL_VERSION(4, 19, 0)) && \
	!((LINUX_VERSION_CODE >= KERNEL_VERSION(4, 14, 0)) && (defined(CHROMIUMOS_KERNEL))))
							  struct device *dev,
#endif
							  struct dma_buf_attachment *attach)
{
	return -ENOSYS;
}

static struct sg_table *gt_pmr_dma_buf_ops_map(struct dma_buf_attachment *attach,
	enum dma_data_direction dir)
{
	return ERR_PTR(-EINVAL);
}

static void gt_pmr_dma_buf_ops_unmap(struct dma_buf_attachment *attach,
	struct sg_table *table, enum dma_data_direction dir)
{
}

static void gt_pmr_dma_buf_ops_release(struct dma_buf *dma_buf)
{
	gt_pmr_unref(dma_buf->priv);
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0))
static void *gt_pmr_dma_buf_ops_kmap(struct dma_buf *dma_buf, unsigned long page_num)
{
	return ERR_PTR(-ENOSYS);
}
#endif

static int gt_pmr_dma_buf_ops_mmap(struct dma_buf *dma_buf, struct vm_area_struct *vma)
{
	return -ENOSYS;
}

static const struct dma_buf_ops gt_pmr_dma_buf_ops = {
	.attach        = gt_pmr_dma_buf_ops_attach,
	.map_dma_buf   = gt_pmr_dma_buf_ops_map,
	.unmap_dma_buf = gt_pmr_dma_buf_ops_unmap,
	.release       = gt_pmr_dma_buf_ops_release,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 12, 0))
#if ((LINUX_VERSION_CODE < KERNEL_VERSION(4, 19, 0)) && \
	!((LINUX_VERSION_CODE >= KERNEL_VERSION(4, 14, 0)) && (defined(CHROMIUMOS_KERNEL))))
	.map_atomic    = gt_pmr_dma_buf_ops_kmap,
#endif
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0))
	.map           = gt_pmr_dma_buf_ops_kmap,
#endif
#else
	.kmap_atomic   = gt_pmr_dma_buf_ops_kmap,
	.kmap          = gt_pmr_dma_buf_ops_kmap,
#endif
	.mmap          = gt_pmr_dma_buf_ops_mmap,
};

bool gt_dma_buf_is_pmr_ops(gt_dma_buf *dmabuf)
{
	return ((struct dma_buf *)dmabuf)->ops == &gt_pmr_dma_buf_ops;
}
GT_EXT_SYM(gt_dma_buf_is_pmr_ops);

gt_dma_buf *gt_pmr_dma_buf_export(void *priv, uint64_t size, int flags)
{
	struct dma_buf *dma_buf;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0))

	DEFINE_DMA_BUF_EXPORT_INFO(dma_export_info);
	dma_export_info.priv  = priv;
	dma_export_info.ops   = &gt_pmr_dma_buf_ops;
	dma_export_info.size  = size;
	dma_export_info.flags = flags;
	dma_buf = dma_buf_export(&dma_export_info);

#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0))
	dma_buf = dma_buf_export(priv, &gt_pmr_dma_buf_ops, size, flags, NULL);
#else
	dma_buf = dma_buf_export(priv, &gt_pmr_dma_buf_ops, size, flags);
#endif
	return dma_buf;
}
GT_EXT_SYM(gt_pmr_dma_buf_export);

int gt_dma_buf_begin_cpu_access(gt_dma_buf *dmabuf, enum gt_dma_data_direction dir)
{
	enum dma_data_direction _dir = gt_get_dma_direction(dir);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 6, 0)) && \
	(!defined(CHROMIUMOS_KERNEL) || (LINUX_VERSION_CODE < KERNEL_VERSION(3, 18, 0)))
	struct dma_buf *buf = (struct dma_buf *)dmabuf;
	return dma_buf_begin_cpu_access(buf, 0, buf->size, _dir);
#else
	return dma_buf_begin_cpu_access(dmabuf, _dir);
#endif
}
GT_EXT_SYM(gt_dma_buf_begin_cpu_access);

int gt_dma_buf_end_cpu_access(gt_dma_buf *dmabuf, enum gt_dma_data_direction dir)
{
	enum dma_data_direction _dir = gt_get_dma_direction(dir);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 6, 0)) && \
	(!defined(CHROMIUMOS_KERNEL) || (LINUX_VERSION_CODE < KERNEL_VERSION(3, 18, 0)))
	struct dma_buf *buf = (struct dma_buf *)dmabuf;
	dma_buf_end_cpu_access(buf, 0, buf->size, _dir);
	return 0;
#else
	return dma_buf_end_cpu_access(dmabuf, _dir);
#endif
}
GT_EXT_SYM(gt_dma_buf_end_cpu_access);

void *gt_dma_buf_vmap(gt_dma_buf *dmabuf)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0))
	return dma_buf_vmap(dmabuf);
#else
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0))
	struct iosys_map map;
#else
	struct dma_buf_map map;
#endif
	dma_buf_vmap(dmabuf, &map);
	return map.vaddr;
#endif
}
GT_EXT_SYM(gt_dma_buf_vmap);

void gt_dma_buf_vunmap(gt_dma_buf *dmabuf, void *addr)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0))
	dma_buf_vunmap(dmabuf, addr);
#else
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0))
	struct iosys_map map;
	iosys_map_set_vaddr(&map, addr);
#else
	struct dma_buf_map map;
	dma_buf_map_set_vaddr(&map, addr);
#endif
	dma_buf_vunmap(dmabuf, &map);
#endif
}
GT_EXT_SYM(gt_dma_buf_vunmap);

void *gt_dma_buf_kmap(gt_dma_buf *dmabuf, unsigned long page_num)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 6, 0))
	return NULL;
#else
	return dma_buf_kmap(dmabuf, page_num);
#endif
}
GT_EXT_SYM(gt_dma_buf_kmap);

void gt_dma_buf_kunmap(gt_dma_buf *dmabuf, unsigned long page_num, void *addr)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 6, 0))
	return;
#else
	dma_buf_kunmap(dmabuf, page_num, addr);
#endif
}
GT_EXT_SYM(gt_dma_buf_kunmap);

int gt_dma_buf_mmap(gt_dma_buf *dmabuf, gt_vm_area *vma, unsigned long pgoff)
{
	return dma_buf_mmap(dmabuf, vma, pgoff);
}
GT_EXT_SYM(gt_dma_buf_mmap);

gt_dma_buf_attachment *gt_dma_buf_attach(gt_dma_buf *dmabuf, void *dev)
{
	return dma_buf_attach(dmabuf, dev);
}
GT_EXT_SYM(gt_dma_buf_attach);

gt_dev* gt_dma_buf_attach_get_dev(gt_dma_buf_attachment *attach)
{
	return ((struct dma_buf_attachment *)attach)->dev;
}
GT_EXT_SYM(gt_dma_buf_attach_get_dev);

void gt_dma_buf_detach(gt_dma_buf *dmabuf, gt_dma_buf_attachment *attach)
{
	dma_buf_detach(dmabuf, attach);
}
GT_EXT_SYM(gt_dma_buf_detach);

gt_sg_table *gt_dma_buf_map_attachment(gt_dma_buf_attachment *attach, enum gt_dma_data_direction dir)
{
	enum dma_data_direction _dir = gt_get_dma_direction(dir);
	return dma_buf_map_attachment(attach, _dir);
}
GT_EXT_SYM(gt_dma_buf_map_attachment);

void gt_dma_buf_unmap_attachment(gt_dma_buf_attachment *attach, gt_sg_table *sg_table, enum gt_dma_data_direction dir)
{
	enum dma_data_direction _dir = gt_get_dma_direction(dir);
	dma_buf_unmap_attachment(attach, sg_table, _dir);
}
GT_EXT_SYM(gt_dma_buf_unmap_attachment);

gt_dma_buf *gt_dma_buf_get_from_attach(gt_dma_buf_attachment *attach)
{
	return ((struct dma_buf_attachment *)attach)->dmabuf;
}
GT_EXT_SYM(gt_dma_buf_get_from_attach);

void gt_get_dma_buf(gt_dma_buf *dmabuf)
{
	get_dma_buf((struct dma_buf *)dmabuf);
}
GT_EXT_SYM(gt_get_dma_buf);

int gt_dma_buf_fd(gt_dma_buf *dmabuf, int flags)
{
	return dma_buf_fd(dmabuf, flags);
}
GT_EXT_SYM(gt_dma_buf_fd);

gt_dma_buf *gt_dma_buf_get(int fd)
{
	return dma_buf_get(fd);
}
GT_EXT_SYM(gt_dma_buf_get);

void gt_dma_buf_put(gt_dma_buf *dmabuf)
{
	dma_buf_put(dmabuf);
}
GT_EXT_SYM(gt_dma_buf_put);

uint64_t gt_dma_buf_size(gt_dma_buf *dmabuf)
{
	return ((struct dma_buf *)dmabuf)->size;
}
GT_EXT_SYM(gt_dma_buf_size);

void *gt_dma_buf_get_priv(gt_dma_buf *dmabuf)
{
	return ((struct dma_buf *)dmabuf)->priv;
}
GT_EXT_SYM(gt_dma_buf_get_priv);

void gt_dma_buf_set_priv(gt_dma_buf *dmabuf, void* priv)
{
	((struct dma_buf *)dmabuf)->priv = priv;
}
GT_EXT_SYM(gt_dma_buf_set_priv);

void gt_dma_cache_sync(void *dev, void *vaddr, uint64_t size, enum gt_dma_data_direction dir)
{
#if defined(CONFIG_LOONGARCH) || defined(CONFIG_MIPS)
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 10, 0))
	enum dma_data_direction _dir = gt_get_dma_direction(dir);
	dma_cache_sync((struct device *)dev, vaddr, size, _dir);
#endif
#endif
}
GT_EXT_SYM(gt_dma_cache_sync);
