/*************************************************************************/ /*!
@File			gtdpu_dp.h
@Title
@Copyright		Copyright (c) Gengtu Technology Ltd. All Rights Reserved
@Description
@License		Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef __GT_DP_H__
#define __GT_DP_H__

#include "gtdpu_common.h"
#include "hal_interface.h"
#include "gtdpu_compatibility.h"
#include "gt_debug.h"

#if (DRM_VERSION >= KERNEL_VERSION(5, 19, 0))
#include <drm/display/drm_dp_helper.h>
#else
#include <drm/drm_dp_helper.h>
#endif

#include "gtdpu_parse_edid.h"
#include "gt_misc.h"
#include "gt_plat_dev.h"
#include "gt_task.h"
#include "gt_lock.h"
#include "gt_timer.h"
#include "gt_mm.h"
#include "gt_waitqueue.h"
#include "gt_drm.h"
#include "gt_drm_mode.h"
#include "gt_fs.h"
#include "gtdpu_dp_common.h"
#include "gtdpu_connector.h"
#include "gtdpu_common_drm_panel.h"

#include <asm/types.h>

struct dp_device_t {
	char *name;
	int zoom_id;
	unsigned int dp_id;
	gt_dev *dev;
	gt_dev *parent;
	struct drm_device *drm_dev;
	struct drm_connector connector;
	struct drm_encoder encoder;
#if (DRM_VERSION >= KERNEL_VERSION(4, 12, 0))
	struct drm_dp_desc desc;
#endif
	struct drm_dp_aux  aux;
	struct drm_dp_aux_msg aux_msg;
	struct backlight_device *bl;
	struct gt_panel *panel;
	struct drm_display_mode current_mode; // pdp->max_mode from current_mode
	struct drm_display_mode native_mode;

	u8 dpcd[DP_RECEIVER_CAP_SIZE];
	u8 edp_dpcd[EDP_DISPLAY_CTL_CAP_SIZE];
	/* sink rates as reported by DP_MAX_LINK_RATE/DP_SUPPORTED_LINK_RATES */
	int max_sink_rates;
	int num_sink_rates;
	int sink_rates[DP_MAX_SUPPORTED_RATES];
	bool use_rate_select;

	gt_workqueue *hpd_wq;
	struct gt_work *hpd_work;

	gt_workqueue *poll_wq;
	struct delayed_work dp_poll_work;
	atomic64_t poll_cnt;

	gt_workqueue *irq_event_wq;
	struct delayed_work irq_event_work;

	gt_mutex *aux_mutex;
	struct audio_conn *ac;
	bool audio_enable;

	struct dp_chip_t chip;

	bool connected;
	bool reduce_lane_count;
	bool hpd_notify;
	bool is_R1;
};

bool gt_dp_is_skip_replace(const struct drm_display_mode *mode);
int gt6910_soc_dp_chip_init(struct dp_chip_t *chip, gt_dev *dev, unsigned int dp_id);
int gt6912_soc_dp_chip_init(struct dp_chip_t *chip, gt_dev *dev, unsigned int dp_id);
int gt6913_soc_dp_chip_init(struct dp_chip_t *chip, gt_dev *dev, unsigned int dp_id);
int gt6908_soc_dp_chip_init(struct dp_chip_t *chip, gt_dev *dev, unsigned int dp_id);

void gt6910_soc_dp_chip_fini(struct dp_chip_t *chip);
void gt6912_soc_dp_chip_fini(struct dp_chip_t *chip);
void gt6913_soc_dp_chip_fini(struct dp_chip_t *chip);
void gt6908_soc_dp_chip_fini(struct dp_chip_t *chip);

int gt6910_soc_dp_audio_sample_bits(u32 sample_bits);
int gt_dp_get_edid_monitor(struct drm_connector *connector);
int sink_lane_status_get(struct dp_device_t *gt_dp, u8 status[DP_LINK_STATUS_SIZE]);
int gt_dp_mvid_calc(struct dp_chip_t *chip);
bool gt_dp_monitor_en(enum dp_monitor_status info);
void gt_dp_check_sink_connection(struct dp_device_t *gt_dp);
int gt_dp_sink_power_ctrl(struct dp_device_t *gt_dp, bool power_on);

#endif
